package org.octopus.data.desensitized.core.springboot.config;

import lombok.extern.slf4j.Slf4j;
import org.octopus.data.desensitized.core.annotation.DesensitizedRule;
import org.octopus.data.desensitized.core.enumeration.DesensitizedPolicy;
import org.octopus.data.desensitized.core.springboot.properties.DesensitizedRuleProperties;
import org.octopus.data.desensitized.core.strategy.AbstractDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.AddressDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.BankCardDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.CarNumberDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.ChineseNameDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.CnapsCodeDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.DefaultDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.EmailDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.FixedPhoneDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.IdCardDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.MobilePhoneDesensitizedStrategy;
import org.octopus.data.desensitized.core.strategy.impl.PasswordDesensitizedStrategy;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author lemon
 * @date 2021/8/28 11:35
 */
@Slf4j
@Configuration
@EnableConfigurationProperties({DesensitizedRuleProperties.class})
public class DesensitizedConfiguration {

    @Bean
    public DefaultDesensitizedStrategy defaultDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.DEFAULT, desensitizedRuleProperties,
                DefaultDesensitizedStrategy.class);
    }

    @Bean
    public AddressDesensitizedStrategy addressDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.ADDRESS, desensitizedRuleProperties,
                AddressDesensitizedStrategy.class);
    }

    @Bean
    public BankCardDesensitizedStrategy bankCardDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.BANK_CARD, desensitizedRuleProperties,
                BankCardDesensitizedStrategy.class);
    }

    @Bean
    public CarNumberDesensitizedStrategy carNumberDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.CARNUMBER, desensitizedRuleProperties,
                CarNumberDesensitizedStrategy.class);
    }

    @Bean
    public ChineseNameDesensitizedStrategy chineseNameDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.CHINESE_NAME, desensitizedRuleProperties,
                ChineseNameDesensitizedStrategy.class);
    }

    @Bean
    public CnapsCodeDesensitizedStrategy cnapsCodeDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.CNAPS_CODE, desensitizedRuleProperties,
                CnapsCodeDesensitizedStrategy.class);
    }

    @Bean
    public EmailDesensitizedStrategy emailDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.EMAIL, desensitizedRuleProperties,
                EmailDesensitizedStrategy.class);
    }

    @Bean
    public FixedPhoneDesensitizedStrategy fixedPhoneDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.FIXED_PHONE, desensitizedRuleProperties,
                FixedPhoneDesensitizedStrategy.class);
    }

    @Bean
    public IdCardDesensitizedStrategy idCardDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.ID_CARD, desensitizedRuleProperties,
                IdCardDesensitizedStrategy.class);
    }

    @Bean
    public MobilePhoneDesensitizedStrategy mobilePhoneDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.MOBILE_PHONE, desensitizedRuleProperties,
                MobilePhoneDesensitizedStrategy.class);
    }

    @Bean
    public PasswordDesensitizedStrategy passwordDesensitizedStrategy(
            DesensitizedRuleProperties desensitizedRuleProperties) {
        return this.obtainDesensitizedStrategy(DesensitizedPolicy.PASSWORD, desensitizedRuleProperties,
                PasswordDesensitizedStrategy.class);
    }

    private <T extends AbstractDesensitizedStrategy<String>> T obtainDesensitizedStrategy(
            DesensitizedPolicy desensitizedPolicy, DesensitizedRuleProperties desensitizedRuleProperties,
            Class<T> desensitizedStrategyClass) {
        DesensitizedRule desensitizedRule = desensitizedRuleProperties.getRules().get(desensitizedPolicy.name());

        try {
            if (desensitizedRule == null) {
                log.info("创建脱敏策略[{}]使用默认规则", desensitizedStrategyClass);
                return desensitizedStrategyClass.newInstance();
            } else {
                log.info("创建脱敏策略[{}]使用规则：{}", desensitizedStrategyClass, desensitizedRule);
                return desensitizedStrategyClass.getConstructor(DesensitizedRule.class).newInstance(desensitizedRule);
            }
        } catch (Exception e) {
            throw new RuntimeException(
                    String.format("创建脱敏策略[%s]失败，脱敏规则：%s", desensitizedStrategyClass, desensitizedRule), e);
        }
    }
}
